/*
 * Example for iqRF click
 *
 * Date         Mar 2017
 * Author       Stefan Popovic
 * Copyright    GNU Public License v2
 *
 * Test configuration STM32 :
 *  MCU           :        STM32F107VCT6
 *  Dev. Board    :        EasyMx PRO v7 for STM32 ARM
 *  SW            :        mikroC PRO for ARM v4.9.1
 *
 *  NOTE          : - This file contains code for both transmitter and receiver.
 *                     It demonstrates sending and receiving of RF packet using 
 *                     UART with 9600 baud rate after given triggering point
 *                     measured with Thermo K sensor 
 *                  - Connect easyTFT and switch SW11.3 into right position
 *                   
 *   TX Setting   : - Place iqRF click board on mikroBUS slot 1
 *                  - Place Thermo K click board on mikroBUS slot 2 
 *                  - If temperature from sensor reaches its triggering point,
 *                     send RF packet to Rx by writing data to UART
 *
 *   RX Setting   : - Place iqRF click board on mikroBUS slot 1                                 
 *                  - Observe alert messages 
 *
 ******************************************************************************/

#define __TX__                          // Choose between __TX__ and __RX__

#include "tft_resources.h"
#include "picture.h"
#include <stdbool.h>

bool low_point_reached = false;
bool high_point_reached = false;

#ifdef __TX__

#include "MCP9600.h"

// Choose triggering temperature points
#define LOW_TEMP_POINT 20
#define HIGH_TEMP_POINT 30

char status;
char txt[10];
char lab1[10];
char old_txt[10];
char tmp_data[10];
char Temp_Data_Ready;

/*******************************************************************************
* Function Write_MCP9600_Data(RegAddr)
* ------------------------------------------------------------------------------
* Overview: Function writes Data to MCP9600
* Input: Register address, data
* Output: Nothing
*******************************************************************************/
char Write_MCP9600_Data(char RegAddr, char _data) 
{
  tmp_data[0] = RegAddr;                                                        // Register address
  tmp_data[1] = _data;
  I2C_Start();                                                                  // Issue I2C start signal
  I2C_Write(MCP9600_I2C_ADDR,tmp_data,2,END_MODE_STOP);                         // Send two bytes (tmp_data[0] and tmp_data[1])
}

/*******************************************************************************
* Function Read_MCP9600_Data(RegAddr)
* ------------------------------------------------------------------------------
* Overview: Function gets Data from MCP9600
* Input: Register address
* Output: Value from desired register
*******************************************************************************/
char Read_MCP9600_Data(char RegAddr)
{
  tmp_data[0] = RegAddr;                                                        // Register address
  I2C_Start();                                                                  // Issue I2C start signal
  I2C_Write(MCP9600_I2C_ADDR,tmp_data,1,END_MODE_RESTART);                      // Send byte (tmp_data[0])
  Delay_us(50);                                                                 // delay 50 us
  I2C_Read(MCP9600_I2C_ADDR,tmp_data,1,END_MODE_STOP);                          // Read register data value and store it in tmp_data
  return tmp_data[0];                                                           // Return MCP9600 Data value
}

/*******************************************************************************
* Function Read_MCP9600_Status
* ------------------------------------------------------------------------------
* Overview: Function gets Data from MCP9600
* Input: Register address
* Output: Value from desired register
*******************************************************************************/
char Read_MCP9600_Status() 
{
  char DataSum;
  tmp_data[0] = MCP9600_Status;                                                 // Register address
  I2C_Start();                                                                  // Issue I2C start signal
  I2C_Write(MCP9600_I2C_ADDR,tmp_data,1,END_MODE_RESTART);                      // Send byte (tmp_data[0])
  Delay_us(50);                                                                 // delay 50 us
  I2C_Read(MCP9600_I2C_ADDR,tmp_data,1,END_MODE_STOP);                          // Read register data value and store it in tmp_data
  DataSum = tmp_data[0];
  return DataSum;                                                               // Return MCP9600 Data value
}

/*******************************************************************************
* Function Read_Temperature()
* ------------------------------------------------------------------------------
* Overview: Function gets Thermocouple Temperature from MCP9600
* Input: Nothing
* Output: Thermocouple Temperature
*******************************************************************************/
float Read_Temperature()
{
  float Temperature;
  tmp_data[0] = MCP9600_TH;                                                     // Thermocouple Temperature register
  I2C_Start();                                                                  // Issue I2C start signal
  I2C_Write(MCP9600_I2C_ADDR,tmp_data,1,END_MODE_RESTART);                      // Send byte (tmp_data[0])
  Delay_us(50);                                                                 // delay 50 us
  I2C_Read(MCP9600_I2C_ADDR,tmp_data,2,END_MODE_STOP);                          // Read thermocouple temperature and store it in tmp_data
  if((tmp_data[0] & 0x80) == 0x80){                                             // TA < 0�C
    tmp_data[0] = tmp_data[0] & 0x7F;                                           // Clear SIGN
    Temperature = 1024 - (tmp_data[0]*16 + tmp_data[1] / 16);
  }
  else                                                                          // TA > 0�C
    Temperature = (tmp_data[0] * 16 + (float)tmp_data[1] / 16);                 // Temperature = Ambient Temperature (�C)
  return Temperature;                                                           // Return Thermocouple temperature data
}

#endif // __TX__
/*******************************************************************************
* Function updateLabel(char *newText, int x, int y, char* lbl)
* ------------------------------------------------------------------------------
* Overview: Function writes text on display and erase old text
* Input: Nothing
* Output: Updated text
*******************************************************************************/
void updateLabel(char *newText, int x, int y, char* lbl)
{
  TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_WHITE, FO_HORIZONTAL);         // Color of background
  TFT_Write_Text(lbl, x, y);                                                    // Write text on TFT display with background color
  strcpy(lbl, newText);                                                         // Copy newText to lbl
  TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);         // Color of text
  TFT_Write_Text(lbl, x, y);                                                    // Write text on  TFT display with black color
}

//******************************************************************************
// Main function
//******************************************************************************

void main() 
{
  char res;
  float temp;
  
  display_init();                                                               // Initialize display
  
  TFT_Image(20, 82, Thermometer_bmp, 1);
  TFT_Image(200, 100, RF_wave_bmp, 1);
  #ifdef __TX__
  // Initialize the I2C1 module with 100Kbps speed on PORTB
  I2C1_Init_Advanced(100000, &_GPIO_MODULE_I2C1_PB67);                          // Initialize I2C
  Delay_ms(200);
  #endif
  UART3_Init_Advanced(9600,                                                     // Initialize UART
                     _UART_8_BIT_DATA, 
                     _UART_NOPARITY, 
                     _UART_ONE_STOPBIT, 
                     &_GPIO_MODULE_USART3_PD89);
  Delay_ms(100);

#ifdef __RX__
////////////////////////////////////////////////////////////////////////////////
  RXNEIE_USART3_CR1_bit = 1;
  NVIC_IntEnable( IVT_INT_USART3 );
  EnableInterrupts();
#endif
////////////////////////////////////////////////////////////////////////////////
   Delay_ms( 2000 );
    UART3_Write_Text("L");
  while(1)
  {
    #ifdef __TX__
    ///////////////////////////////////////////////////////////////////////////
    status = &Read_MCP9600_Status;
    Temp_Data_Ready = (status & 0x40);

    temp =  Read_Temperature();
    
    if (Temp_Data_Ready)
    {                                                                           // Check is Temperature Data ready
      sprintf(txt, "%2.1f  C", temp);                                           // Format Temperature Data and store it to txt
      res = strcmp(txt,old_txt);                                                // Compare old_txt and txt

      if(res != 0)                                                              // If old_txt and txt are not equal
      {                                                                         
        updateLabel(txt, 135, 90, lab1);                                        // Write temperature value on display
        strcpy(old_txt,txt);                                                    // Copy txt to old_txt string array
      }
    }
    
    if (temp <= LOW_TEMP_POINT)                                                 // If low temperature point has been reached
    {
     if (!low_point_reached)
     {
      UART3_Write_Text("L");                                                    // Send byte via UART
     }
     low_point_reached = true;
       
    }
    else if (temp >= HIGH_TEMP_POINT)                                           // If high temperature point has been reached
    {
     if(!high_point_reached)
     {
      UART3_Write_Text("H");                                                    // Send byte via UART                                 
     }
     high_point_reached = true;

    }
    else
    {
     low_point_reached = false;
     high_point_reached = false;
    }
    Delay_ms(500);
  #endif
  //////////////////////////////////////////////////////////////////////////////
  #ifdef __RX__

    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);

    // Writing warnings on display
    if(high_point_reached)
    {
     TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
     high_point_reached = false;
     TFT_Write_Text("HIGH ", 130, 100);
     Delay_ms(2000);
     TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_WHITE, FO_HORIZONTAL);
     TFT_Write_Text("HIGH ", 130, 100);
    }
    if(low_point_reached)
    {
     TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLUE, FO_HORIZONTAL);
     low_point_reached = false;
     TFT_Write_Text("LOW", 130, 100);
     Delay_ms(2000);
     TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_WHITE, FO_HORIZONTAL);
     TFT_Write_Text("LOW ", 130, 100);
    }
  #endif
  }

}

#ifdef __RX__
////////////////////////////////////////////////////////////////////////////////
/*******************************************************************************
* Function iqRF_RX_ISR()
* ------------------------------------------------------------------------------
* Overview: UART Interrupt InService Routine for receiving UART data from IQRF MCU
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void iqRF_RX_ISR() iv IVT_INT_USART3 ics ICS_AUTO
{
    if( RXNE_USART3_SR_bit )
    {
        char tmp = USART3_DR;

        if( tmp == 'L')
        {
         low_point_reached = true;
         high_point_reached = false;
        }
        else if( tmp == 'H')
        {
         high_point_reached = true;
         low_point_reached = false;
        }
    }
}
#endif
//////////////////////////////////////////////////////////////////END OF FILE///